@@ -2,6 +2,7 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
+#include <string.h>
 #include <ctype.h>
 #include <unistd.h>
 #include "png.h"
@@ -9,7 +10,6 @@
 #include "config.h"
 
 int verbose;
-int sng_error;
 int idat;
 
 png_struct *png_ptr;
@@ -113,7 +113,8 @@ void initialize_hash(int hashfunc(color_item *),
 
 	    for (;;)
 	    {
-		fgets(line, sizeof(line) - 1, fp);
+		if (fgets(line, sizeof(line) - 1, fp) == NULL)
+		    break;
 		st = sscanf(line, "%d %d %d %[^\n]\n", 
 			     &red, &green, &blue, namebuf);
 		if (feof(fp))
@@ -152,9 +153,15 @@ void initialize_hash(int hashfunc(color_item *),
  *
  ************************************************************************/
 
+static int max(int x, int y)
+{
+    return x > y ? x : y;
+}
+
 int main(int argc, char *argv[])
 {
     int i = 1;
+    int error_status = 0;
 
 #ifdef __EMX__
     _wildcard(&argc, &argv);   /* Unix-like globbing for OS/2 and DOS */
@@ -172,7 +179,7 @@ int main(int argc, char *argv[])
 	    ++verbose;
 	    i++;
 	    break;
-	case 'i':
+	case 'i':    /* dump raw IDAT chunks - unimplemented, undocumented */
 	    ++idat;
 	    i++;
 	    break;
@@ -213,6 +220,7 @@ int main(int argc, char *argv[])
 	    if (argv[i][dot] != '.')
 	    {
 		fprintf(stderr, "sng: %s is neither SNG nor PNG\n", argv[i]);
+		error_status = max(error_status, 1);
 		continue;
 	    }
 	    else if (strcmp(argv[i] + dot, ".sng") == 0)
@@ -232,6 +240,7 @@ int main(int argc, char *argv[])
 	    else
 	    {
 		fprintf(stderr, "sng: %s is neither SNG nor PNG\n", argv[i]);
+		error_status = max(error_status, 1);
 		continue;
 	    }
 
@@ -243,24 +252,24 @@ int main(int argc, char *argv[])
 		fprintf(stderr,
 			"sng: couldn't open %s for input (%d)\n",
 			argv[i], errno);
+		error_status = max(error_status, 1);
 		continue;
 	    }
 	    if ((fpout = fopen(outfile, "w")) == NULL)
 	    {
 		fprintf(stderr,
-			"sng: couldn't open for output %s (%d)\n",
+			"sng: couldn't open %s for output (%d)\n",
 			outfile, errno);
+		error_status = max(error_status, 1);
 		continue;
 	    }
 
 	    if (sng2png)
-		sngc(fpin, argv[i], fpout);
+		error_status = max(error_status, sngc(fpin, argv[i], fpout));
 	    else
-		sngd(fpin, argv[i], fpout);
+		error_status = max(error_status, sngd(fpin, argv[i], fpout));
 	}
     }
 
-    /* This only returns the error on the last file.  Works OK if you are only
-     * checking the status of a single file. */
-    return sng_error;
+    return error_status;
 }
